<html>
<head>

</head>
 
<body>
    <!-- Content for Static Frame 'header_frame' -->
    <div id="header_content"><p style="font-size: 9px; color: grey">PROJECT NAME     DATASET NAME    MODEL NAME </p></div>
    <div id="header_content"><p style="font-size: 18px;"><b>{{ TITLE }}</b></p></div>
    <div id="header_content"><p style="font-size: 18px;">{{ FIGURES }}</p></div>
    <div id="header_content"><p style="font-size: 18px;">{{ BODY }}</p></div>
 
    <!-- HTML Content -->
    {{ FIGURES }}
</body>
</html>